"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parsers = {
    json: JSON.parse,
    json5: function (source) {
        return require('json5').parse(source);
    },
    yaml: function (source) {
        return require('js-yaml').safeLoad(source);
    }
};
function compileTarget(source, CodeBuffer, options) {
    const { format, getNamespace } = options;
    const buffer = new CodeBuffer({
        root: options.rootVar,
        filePath: options.filePath
    });
    let json;
    try {
        json = parsers[format](source);
    }
    catch (e) {
        console.error(`json parse error: ` + e.message);
        return {
            code: '',
            errors: [{
                    code: 1,
                    message: 'json parse error: ' + e.message
                }]
        };
    }
    if ('$title' in json) {
        delete json.$title;
    }
    if ('$preprocesser' in json) {
        const preprocesser = json.$preprocesser;
        buffer.addPreprocesser(preprocesser, getNamespace);
        delete json.$preprocesser;
    }
    buffer.walk(json);
    const code = buffer.toString();
    return {
        code,
        errors: []
    };
}
exports.compileTarget = compileTarget;
//# sourceMappingURL=common.js.map