"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("./js/index");
const index_2 = require("./php/index");
const compilers = {
    js: index_1.compile,
    php: index_2.compile
};
function compile(options) {
    let errors = [];
    let code = {};
    const { source, rootVar, format = 'json', filePath, target = { php: true, js: true }, getNamespace } = options;
    if (!Object.keys(target).find(key => target[key])) {
        console.error('please specify at least one target');
        return {
            code: {},
            errors: []
        };
    }
    Object.keys(target).forEach(key => {
        if (target[key] && compilers[key]) {
            const res = compilers[key]({
                source,
                rootVar,
                format,
                filePath,
                getNamespace
            });
            code[key] = res.code;
            res.errors && errors.push(...res.errors);
        }
    });
    return {
        code,
        errors
    };
}
exports.default = compile;
//# sourceMappingURL=compiler.js.map