"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../utils");
const json2js_1 = __importDefault(require("./json2js"));
var CodeType;
(function (CodeType) {
    CodeType[CodeType["line"] = 0] = "line";
    CodeType[CodeType["raw"] = 1] = "raw";
})(CodeType || (CodeType = {}));
const U = 'legoMirrorUtil';
const tracer = 'ijkhlmn';
class CodeBuffer {
    constructor(options) {
        this.buffer = [{
                type: CodeType.line,
                code: `var $newData = {}`
            }];
        this.trackerIndex = 0;
        this.root = options.root || '$tplData';
        this.filePath = options.filePath;
    }
    walk(mirror, parentPath) {
        for (let [to, from] of Object.entries(mirror)) {
            const toPath = utils_1.getPath(to);
            if (typeof from === 'string') {
                if (utils_1.isPointer(from)) {
                    let fromPath = utils_1.getFromPath(from, parentPath);
                    this.fromPath(fromPath, toPath, parentPath);
                }
                else if (utils_1.isMustache(from)) {
                    this.fromMustache(from, toPath, parentPath);
                }
                else {
                    this.fromConstant(from, toPath, parentPath);
                }
            }
            else if (['number', 'boolean'].includes(typeof from)
                || Array.isArray(from)
                || !from.$from) {
                this.fromConstant(from, toPath, parentPath);
            }
            else if (typeof from === 'object' && from.$from) {
                const { $from, $action = 'copy', $data } = from;
                switch ($action) {
                    case 'copy':
                        if (typeof $data === 'object') {
                            this.addCopy($from, toPath, $data, parentPath);
                        }
                    default:
                        break;
                }
            }
        }
    }
    addCopy(from, to, data, parentPath) {
        const { $type = 'object', $mirror } = data;
        if (!$mirror) {
            utils_1.isPointer(from)
                ? this.fromPath(utils_1.getPath(from), to, parentPath)
                : this.fromConstant(from, to, parentPath);
            return;
        }
        if (!parentPath) {
            parentPath = { from: [], to: [] };
        }
        if ($type === 'object' && utils_1.isPointer(from)) {
            let fromPath = utils_1.getFromPath(from, parentPath);
            const { setter, getter } = this.getParams({ from: fromPath, to });
            this.buffer.push({
                type: CodeType.raw,
                code: `if (${getter} !== undefined) {\nif (${setter} === undefined) {`
            });
            this.fromConstant({}, to, parentPath);
            this.buffer.push({
                type: CodeType.raw,
                code: `}`
            });
            this.walk($mirror, {
                from: [...parentPath.from, ...fromPath],
                to: [...parentPath.to, ...to]
            });
            this.buffer.push({
                type: CodeType.raw,
                code: `}`
            });
            return;
        }
        if ($type === 'object') {
            this.fromConstant(from, to, parentPath);
            return;
        }
        if ($type === 'array') {
            let fromPath = utils_1.getFromPath(from, parentPath);
            const { setter, getter } = this.getParams({
                from: [...parentPath.from, ...fromPath],
                to: [...parentPath.to, ...to
                ]
            });
            this.buffer.push({
                type: CodeType.raw,
                code: `if (${getter} !== undefined) {\nif (Object.prototype.toString.call(${getter}) !== '[object Array]') {\n${getter} = [${getter}];\n}`
            });
            if (data.$maxItems) {
                this.buffer.push({
                    type: CodeType.raw,
                    code: `${getter} = ${getter}.slice(0, ${data.$maxItems});`
                });
            }
            const tracerVar = `$${tracer[this.trackerIndex]}`;
            this.trackerIndex++;
            this.buffer.push({
                type: CodeType.raw,
                code: `${getter}.forEach(function($item, ${tracerVar}) {\nif (${setter}[${tracerVar}] === undefined) {`
            });
            this.fromConstant({}, [...to, tracerVar], parentPath);
            this.buffer.push({
                type: CodeType.raw,
                code: `}`
            });
            this.walk($mirror, {
                from: [...parentPath.from, ...fromPath, tracerVar],
                to: [...parentPath.to, ...to, tracerVar]
            });
            this.buffer.push({
                type: CodeType.raw,
                code: `})\n}`
            });
        }
    }
    fromConstant(constant, to, parentPath) {
        const { setter } = this.getParams(parentPath);
        let p = {
            type: CodeType.line,
            code: this.transfromSetter(to, setter, json2js_1.default(constant))
        };
        this.buffer.push(p);
    }
    fromMustache(from, to, parentPath) {
        const { getter, setter } = this.getParams(parentPath);
        let regExp = /{{\s*#\/.+?(?=}})/g;
        let varArray = from.match(regExp);
        let array = [];
        let tmpStr = from;
        for (let i = 0; i < varArray.length; i++) {
            let strArray = tmpStr.split(varArray[i]);
            array.push({
                type: 'const',
                val: strArray[0].replace('}}', '')
            });
            array.push({
                type: 'val',
                val: varArray[i].replace('{{', '').trim()
            });
            tmpStr = strArray[1];
        }
        let lastConst = tmpStr.replace('}}', '');
        if (lastConst) {
            array.push({
                type: 'const',
                val: lastConst
            });
        }
        let code = '';
        for (let i = 0; i < array.length; i++) {
            let item = array[i];
            if (item.type === 'val') {
                let valCode = this.transformGetter(utils_1.getPath(item.val), getter);
                code += valCode;
            }
            else {
                code += json2js_1.default(item.val);
            }
            if (i !== array.length - 1) {
                code += ' + ';
            }
        }
        let p = {
            type: CodeType.line,
            code: this.transfromSetter(to, setter, code)
        };
        this.buffer.push(p);
    }
    fromPath(from, to, parentPath) {
        const { getter, setter } = this.getParams(parentPath);
        let p = {
            type: CodeType.line,
            code: this.transfromSetter(to, setter, this.transformGetter(from, getter))
        };
        this.buffer.push(p);
    }
    getParams(parentPath) {
        let getter = this.root;
        let setter = '$newData';
        if (parentPath && parentPath.from.length > 0) {
            getter = this.transformGetter(parentPath.from, getter);
        }
        if (parentPath && parentPath.to.length > 0) {
            setter = this.transformGetter(parentPath.to, setter);
        }
        return { getter, setter };
    }
    transformGetter(path, variable) {
        let p = `${variable}${path.map(p => `[${/^\$/.test(p) ? p : json2js_1.default(p)}]`).join('')}`;
        return p;
    }
    transfromSetter(to, setter, content) {
        return `${U}.set(${setter}, [${to.map(t => /^\$/.test(t) ? t : json2js_1.default(t)).join(', ')}], ${content})`;
    }
    addPreprocesser(preprocesser, getNamespace) {
        this.buffer.push({
            type: CodeType.raw,
            code: `\
require_once(dirname(__FILE__) . '/' . ${JSON.stringify(preprocesser.replace(/\.(ts|php)$/, '') + '.php')});
${this.root} = array_merge(${this.root}, ${getNamespace(preprocesser)}\\process(${this.root}));`
        });
    }
    toString() {
        return this.buffer
            .map(a => a.code + (a.type === CodeType.line ? ';' : ''))
            .join('\n');
    }
}
exports.default = CodeBuffer;
//# sourceMappingURL=CodeBuffer.js.map